/**
 * 
 */
import org.w3c.dom.*;
import org.xml.sax.*;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import java.io.FileOutputStream;
import java.io.*;

public class TestAAD2 {
    DOMParser parseAD;
    Document ADdoc;
    InputStream is=null;

    /**------------------------------------------------------------
       Parse the input file to make sure it is valid, well-formed.
       This constructor assumes the XML file exists as an
       external document. Will need another constructor if xml file
       is passed internally from another program.
    */
    public TestAAD2(String xmlFileName){
	// 
	// 
	// 
	try{
	    parseAD=new DOMParser();
	    parseAD.parse(xmlFileName);
	    ADdoc=parseAD.getDocument();
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
	
    }
    
    /**------------------------------------------------------------
       Returns a string array with all of the tag values for 
       a particular tag name, in order that they appear in the
       xml document.  For instance, if a tag appears 7 times,
       the returned String array will have length 7, with the
       CDATA values of the tags stored as strings.

       Example Usage: array=getTagValueArray(ADdoc, "inputParam");
    */
    public String[] getTagValueArray(Document doc,String tagName){
	Node node;
	NodeList nl=doc.getElementsByTagName(tagName);
	if(nl==null) {
	    System.err.println("Entered unknow tag");
	    System.exit(0);
	}
	String [] nodeValues=new String[nl.getLength()];
	for (int i=0; i<nl.getLength();i++) {
	    node=nl.item(i);
	    nodeValues[i]=node.getFirstChild().getNodeValue();
	}
	return nodeValues;
    }
    
    /**------------------------------------------------------------
       For a given tag, tagName, and a given attribute named attName,
       returns the value of the attribute.  For now I assume the 
       tag occurs only once in the xml doc.
    */
    public String getTagAttribValue(Document doc, String tagName, String attName ) {
	NodeList nl=doc.getElementsByTagName(tagName);
	if(nl==null) {
	    System.err.println("Entered unknow tag");
	    System.exit(0);
	}
	Element elem=(Element)nl.item(0);  //Assume only one instance of tag.
	if(elem.getAttribute(attName)==null) {
	    System.err.println("Entered unknown attribute");
	    System.exit(0);
	}	    
	return elem.getAttribute(attName);
    }
    
    /**------------------------------------------------------------
       This method creates the PBS script from the xml file.  Can make
       this more general by defining a tag with value "PBS" or "NQS" or
       whatever.
    */
    public void createPBS(String PBSfile) {
	//Set up printing
	File f=null;
	PrintWriter out=null;
	try{
	    f=new File(PBSfile);
	    out=new PrintWriter(new FileWriter(f));
	}
	catch(IOException e) {e.printStackTrace();}
	
	/* This section lists all the PBS directives */
    	out.println("#! /bin/csh");
	if(getTagAttribValue(ADdoc,"exportEnv","export").equals("Yes"))
	    out.println("#PBS -V");

	out.println("#PBS "+ getTagAttribValue(ADdoc,"queueType","flag")
		    +" "+getTagValueArray(ADdoc,"queueType")[0]);
	out.println("#PBS "+ getTagAttribValue(ADdoc,"shell","flag")+" "
		    +getTagValueArray(ADdoc,"shell")[0]);
	out.println("#PBS "+ getTagAttribValue(ADdoc,"jobName","flag")+" "
		    +getTagValueArray(ADdoc,"jobName")[0]);
	out.println("#PBS "+ getTagAttribValue(ADdoc,"NNodes","flag")+" "
		    +getTagValueArray(ADdoc,"NNodes")[0]);
	out.println("#PBS "+ getTagAttribValue(ADdoc,"walltime","flag")+" "
		    +getTagValueArray(ADdoc,"walltime")[0]);
	out.println("#PBS "+ getTagAttribValue(ADdoc,"SystemOutFile","flag")+" "
		    +getTagValueArray(ADdoc,"SystemOutFile")[0]);
	out.println("#PBS "+ getTagAttribValue(ADdoc,"stderr","flag")+" "
		    +getTagValueArray(ADdoc,"stderr")[0]);
      
	/* Now print out the shell directives */
	String workspace=getTagValueArray(ADdoc,"workdir")[0];
        out.println("if( ! -d "+workspace +" then");
        out.println("  mkdir -p "+workspace);
        out.println("else");
        out.println("exit 1");
        out.println("endif");
	out.println("cd "+workspace);
  	out.println("cp "+getTagValueArray(ADdoc,"dataindir")[0]+"/"
  		    +getTagValueArray(ADdoc,"inputFile")[0] + " "
  		    +workspace+getTagValueArray(ADdoc,"inputFile")[0]
		    );
	
	String InSeparator=null;
	String iostyle=getTagAttribValue(ADdoc,"IOStyle","style");
	if(iostyle.equals("CArgument")) {
	    InSeparator=" ";
	}
	else if (iostyle.equals("StandardIO")) {
	    InSeparator=" < ";
	}
	else { 
	    System.out.println("Invalid value for IOStyle attribute");
	    System.exit(0);
	}
		

	//Cat a big string for the executable path
	String execstuff=getTagValueArray(ADdoc,"execPath")[0];
	for(int i=0;i<getTagValueArray(ADdoc,"inputParam").length;i++) {
	    execstuff+=InSeparator;
	    execstuff+=getTagValueArray(ADdoc,"inputParam")[i];
	}
	out.println(execstuff);
	
	//Loop over output files and cp back to users directory
	for(int i=0;i<getTagValueArray(ADdoc,"outputFile").length;i++) {
	    out.println("cp "+workspace+" "+getTagValueArray(ADdoc,"outputFile")[i]);
	}

	//We are done.  
  	out.close();
    }
    
     public static void main(String[] args) {
	if(args.length!=1) {
	    System.out.println("Usage: java TestAAD2 <xml file>");
	    System.exit(0);
	}
	TestAAD2 test=new TestAAD2(args[0]);
	test.createPBS("test.pbs");
    }
   
}




