/**
 * Program Name: parseXMLBean.java
 * Comments: Several convenience methods for manipulating the ApplDesc.xml file
 * This Bean may be converted into a webflow module later.
 */

package generic;

import java.util.*;
import org.w3c.dom.*;
import org.xml.sax.*;
import org.apache.xerces.parsers.DOMParser;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.ErrorHandler;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.apache.xml.serialize.XMLSerializer;
import org.apache.xml.serialize.OutputFormat;
import java.io.*;

public class parseXMLBean {
    DOMParser parseAD;
    Document ADdoc;
    InputStream is=null;

    /**------------------------------------------------------------
     * Parse the input file to make sure it is valid, well-formed.
     *This constructor assumes the XML file exists as an
     * external document. Will need another constructor if xml file
     * is passed internally from another program.
     */
    public parseXMLBean(){
    }
    
    public void init(String xmlFileName) {
	try{
	    parseAD=new DOMParser();
	    parseAD.parse(xmlFileName);
	    ADdoc=parseAD.getDocument();
	}
	catch (Exception ex) {
	    ex.printStackTrace();
	}
    }
    
    /**------------------------------------------------------------
     * Returns a string array with all of the tag values for 
     * a particular tag name, in order that they appear in the
     * xml document.  For instance, if a tag appears 7 times,
     * the returned String array will have length 7, with the
     * CDATA values of the tags stored as strings.
     *
     * Example Usage: array=getTagValueArray("inputParam");
     *------------------------------------------------------------*/
    public String[] getTagValueArray(String tagName){
	Node node;
	NodeList nl=ADdoc.getElementsByTagName(tagName);
	if(nl==null) {
	    System.err.println("Entered unknow tag");
	    System.exit(0);
	}
	String [] nodeValues=new String[nl.getLength()];
	for (int i=0; i<nl.getLength();i++) {
	    node=nl.item(i);
	    nodeValues[i]=node.getFirstChild().getNodeValue();
	}
	return nodeValues;
    }
    
    /**------------------------------------------------------------
     * For a given tag, tagName, and a given attribute named attName,
     * returns the value of the attribute.  For now I assume the 
     * tag occurs only once in the xml doc.
     *------------------------------------------------------------*/
    public String getTagAttribValue(String tagName, String attName ) {
	NodeList nl=ADdoc.getElementsByTagName(tagName);
	if(nl==null) {
	    System.err.println("Entered unknow tag");
	    System.exit(0);
	}
	Element elem=(Element)nl.item(0);  //Assume only one instance of tag.
	if(elem.getAttribute(attName)==null) {
	    System.err.println("Entered unknown attribute");
	    System.exit(0);
	}	    
	return elem.getAttribute(attName);
    }
    
    /**------------------------------------------------------------
     * For a given tag name that occurs multiple times, return a
     * string array that contains the values of it's attributes, in
     * order that they appear in the file xmlFileName.  For example,
     * in the ApplDesc.xml file, the tag "application" occurs multiple
     * times, each with an attribute called "id" that gives the name
     * of the code (i.e., Gaussian, parssim, etc.).  These code names
     * would be returned in a string array.
     * 
     * Ex: array=this.getTagAttribValues("application","id");
     *------------------------------------------------------------*/
    public String[] getTagAttribValues(String tagName, String attName) {
	Node node;
	Element elem;
	NodeList nl=ADdoc.getElementsByTagName(tagName);
	if(nl==null) {
	    System.err.println("Entered unknow tag");
	    System.exit(0);
	}
	String[] nodeAttValues=new String[nl.getLength()];
	for (int i=0;i<nl.getLength();i++) {
	    elem=(Element)nl.item(i);
	    nodeAttValues[i]=elem.getAttribute(attName);
	}
	return nodeAttValues;
    }

    /**
     * This specialized method returns the available hosts for a particular 
     * machine.  Must provide the value of the application id.  
     * The id's of the 
     * <target> children of the matching tag are then returned if they
     * are installed.
     */
    public Vector getAllowedHosts(String appIdVal) {
	//I will assume the tagName can be hardset to "application".
	String tagName="application";
	String attName="id";
	Vector allowedHosts=new Vector();
	Element elem;
	Node appNode=null;
	NodeList nl1,nl2;
	nl1=ADdoc.getElementsByTagName(tagName);
	if(nl1==null) {
	    System.err.println("Entered unknow tag");
	    System.exit(0);
	}

	//Find the application tag with the correct id.
	int i=0;
	boolean test=true;
	while(test) {
	    elem=(Element)nl1.item(i);
	    if(elem.getAttribute(attName).equals(appIdVal)) {
		appNode=nl1.item(i);
		test=false;
	    }
	    else {
		i++;
	    }
	}
	
	//Now get the children named "target" of the appNode.
	nl2=appNode.getChildNodes();
	Node targNode,statusNode;
	NodeList nl3;
	Element targElem,statusElem;
	for(int j=0;j<nl2.getLength();j++) {
    	    targNode=nl2.item(j);
    	    if((targNode.getNodeName()).equals("target")) {
		System.out.println("Look in target");
    	 	nl3=targNode.getChildNodes();
		for(int k=0;k<nl3.getLength();k++){
		    statusNode=nl3.item(k); 
		    if(statusNode.getNodeName().equals("status")){
			statusElem=(Element)nl3.item(k);
			System.out.println(statusElem.getAttribute("installed"));
			if(statusElem.getAttribute("installed").equals("Yes")) {
			    System.out.println("Code is installed");
			    System.out.println(((Element)targNode).getAttribute("id"));
			    allowedHosts.add(((Element)targNode).getAttribute("id"));
			}
		    }
		}
	    }
	}
	return allowedHosts;
    }
    /**
     * This specialized method returns the values of the child nodes
     * of the <installed> tag for a given target.  To find the value of
     * <execpath>, for example, you would use something like
     * getInstalledVal("ising","dev02n03.asc.hpc.mil","execpath").
     */
    public String getInstalledVal(String appID,String targID,String ChildTag) {
	NodeList nl1, nl2,nl3;
	Node appNode=null,targNode=null;
	Element elem;
	String ChildTagVal="";
	boolean test;
	int i;
	nl1=ADdoc.getElementsByTagName("application");
	
	//Find the application tag with the correct id.
	i=0;
	test=true;
	while(test) {
	    elem=(Element)nl1.item(i);
	    System.out.println(elem.getAttribute("id"));
	    if(elem.getAttribute("id").equals(appID)) {
		appNode=nl1.item(i);
		System.out.println("Appnode is "+appNode.getNodeName());
		test=false;
	    }
	    else {
		i++;
	    }
	}
	
	//Now get the target tag with the correct id.
	nl2=((Element)appNode).getElementsByTagName("target");
	i=0;
	test=true;
	while(test) {
	    System.out.println(i+" "+nl2.item(i).getNodeName());
	    elem=(Element)nl2.item(i);
	    if(elem.getAttribute("id").equals(targID)) {
		targNode=nl2.item(i);
		test=false;
	    }
	    else {
		i++;
	    }
	}
	
	//Now get value of the specified child tag
	nl3=((Element)targNode).getElementsByTagName("installed");
	System.out.println("nl3 length is "+nl3.getLength());
	for(int k=0;k<nl3.getLength();k++) 
	    System.out.println(k+" "+nl3.item(k).getNodeName());
	NodeList nl4=((Element)nl3.item(0)).getElementsByTagName(ChildTag);
	i=0;
        test=true;
	System.out.println("nl4 length is " + nl4.getLength());
	for(int j=0;j<nl4.getLength();j++){
	    System.out.println(j+" "+nl4.item(j).getNodeName());
	}
	while(test) {
	    System.out.println("Child is "+nl4.item(i).getNodeName()
			       +" "+nl4.item(i).getFirstChild().getNodeValue());
	    if((nl4.item(i).getNodeName()).equals(ChildTag)) {
		ChildTagVal=nl4.item(i).getFirstChild().getNodeValue();
		test=false;
	    }
	    else {
		i++;
	    }
	}
	return ChildTagVal;
    }
    
}




